function [w, it, cmval, muopt, W] = os_cma(X, gamma, wini, tol, maxit)

% Iterative blind channel equalization and signal extraction via the
% optimal step-size constant modulus algorithm (OS-CMA).
%
% Batch processing implementation: all samples in the received data frame are used
% at each iteration of the algorithm.
%
%
% SYNTAX: [w, it, cmval, muopt, W] = os_cma(X, gamma, wini, tol, maxit);
%
% OUTPUTS:  
%         w     : equalizer vector by the OS-CMA algorithm after convergence
%      
%         it    : number of iterations for convergence.
% 
%         cmval : CM cost value at each iteration;
%                 cmval(1) contains the CM cost value before starting the algorithm
%
%         muopt : optimal step size at each iteration
%
%         W     : equalizer vectors as a function of iteration number
%                 (one iteration per column; first column: initial extracting vector).
%
%
% INPUTS: 
%         X     : observed data matrix;
%                 a (block) Toeplitz matrix in the case of time equalization
%         
%         gamma : constellation-dependent constant used in CM criterion;
%                 this constant is equal to E{|s^4|}/E{|s^2|}, where s is the source;
%                 (e.g., gamma = 1 for PSK modulations)
%
%         wini  : initial value of the equalizer vector;
%                 column vector with the same length as the number of rows in X
%
%         tol   : termination-test threshold;
%                 the algorithm terminates if
%                    ||w' - w||/||w|| < tol/sqrt(T)
%                 where w' and w are the equalizer vectors at the current and
%                 previous iteration, respectively, and the current  T is the
%                 data block size (number of columns in X)
%
%        maxit  : maximum number of iterations.
%
%
% REFERENCES:
%
% - V. Zarzoso, P. Comon and D. Slock, <a href = "http://www.elsevier.com/books/handbook-of-blind-source-separation/comon/978-0-12-374726-6#description">"Semi-Blind Methods for Communications"</a>,
%   in: P. Comon and C. Jutten (Eds.), Handbook of Blind Source Separation, Independent Component Analysis and
%   Applications, Oxford, UK: Academic Press, 2010, chapter 15, pp. 593-638.
%
% - V. Zarzoso and P. Comon, <a href = "http://www.i3s.unice.fr/~zarzoso/biblio/tcom08.pdf">"Optimal Step-Size Constant Modulus Algorithm"</a>,
%   IEEE Transactions on Communications, Vol. 56, No. 1, January 2008, pp. 10-13.
%
% - V. Zarzoso et P. Comon, <a href = "http://www.lavoisier.fr/notice/fr2746215180.html">"Methodes semi-aveugles pour les telecommunications"</a>,
%   dans : C. Jutten et P. Comon, Separation de sources 2 : au-dela de l'aveugle et applications,
%   Paris : Hermes, 2007, chapitre 8, pp. 303-353.
%
% - V. Zarzoso and P. Comon, <a href = "http://www.i3s.unice.fr/~zarzoso/biblio/icassp05.pdf">"Semi-Blind Constant Modulus Equalization with Optimal Step Size"</a>,
%   in: Proceedings ICASSP-2005, 30th IEEE International Conference on Acoustics, Speech and Signal Processing,
%   Philadelphia, PA, March 18-23, 2005, Vol. III, pp. 577-580. 
%
% - V. Zarzoso and P. Comon, <a href = "http://www.i3s.unice.fr/~zarzoso/biblio/eusipco05.pdf">"Blind Channel Equalization with Algebraic Optimal Step Size"</a>,
%   in: Proceedings EUSIPCO-2005, XIII European Signal Processing Conference, Antalya, Turkey, September 4-8, 2005.
%
%
% Please, report any bugs, comments or suggestions to <a href = "mailto:zarzoso@i3s.unice.fr">zarzoso(a)i3s.unice.fr</a>.
%
%
% HISTORY:
% 
%    <Please add modification date here>: - <please add modification details here>
%         
% -- 2014/11/21: Version 1 release ----------------------------------------------------------------
%
%    2010/02/18: - created by Vicente Zarzoso
%                  (I3S Laboratory, University of Nice Sophia Antipolis, CNRS, France).


[L, T] = size(X);

tol = tol/sqrt(T);  % a statistically-significant termination threshold 

w = wini;           % initialize the extracting vector

cmval = zeros(1, maxit+1);  % CM cost function values as a function of iteration
muopt = zeros(1, maxit);    % opt step size values a a function of iteration
W = zeros(L, maxit+1);      % extracting vectors as a function of iteration

it = 0;
keep_going = 1;

while keep_going
    
    it = it + 1;

    cmval(it) = cm_cost(w, X, gamma);       % current value of CM cost
    W(:, it) = w;                           % current extracting vector
        
    y = w'*X;
    g = X*(y.*(abs(y).^2 - gamma))'/T;  % CM gradient at w (up to irrelevant constant factor)
    
    norm_g = norm(g);
    
    if norm_g > eps 
       g = g/norm_g;    % normalize the gradient -> parameter of interest: direction 
                        % improves conditioning of opt step-size polynomial  
       mu_opt = cm_optstep(w, g, X, gamma);
    else
       mu_opt = 0;  
    end 
   
    muopt(it) = mu_opt; % store optimal step size at current iteration
    
    % update extracting vector
    wn = w - mu_opt*g;  % perform gradient descent on CM cost function using optimal step size
 
    % compute convergence parameter
    th = norm(wn-w)/norm(w);
    
    w = wn;
    
    if th < tol || it >= maxit || any(isnan(wn))  
        % finish when convergence, divergence or too many iterations
        keep_going = 0;     
    end %if th < tol ...
    
end % while keep_going 

cmval(it+1) = cm_cost(w, X, gamma); % compute final CM-cost value
W(:, it+1) = w;                     % store final extracting vector




function mu_opt = cm_optstep(w, g, X, gamma)

% Computes optimal step size for the CM criterion.
%
%
% SYNTAX: mu_opt = cm_optstep(w, g, X, gamma); 
%
%         mu_opt : optimal step size globally minimizing the CM criterion
%                  along direction g from f;
%                  returns zero in case of degeneracy
%
%         w      : extracting filter coefficients
%         g      : search direction (e.g., gradient) at w 
%         X      : data matrix
%         gamma  : constellation-dependent constant used in CM criterion.


%%% Compute optimal step-size polynomial coefficients

yw = w'*X;
yg = g'*X;
  
% interim coefficients
a = abs(yg).^2;
b = -2*real(yw.*conj(yg));
c = abs(yw).^2 - gamma;

% work out polynomial coefficients
pol = real([2*mean(a.^2), 3*mean(a.*b), mean(2*a.*c + b.^2), mean(b.*c)]);

%%% Obtain roots
rts = roots(pol); 
                            
% check degeneracy
if isempty(rts)
   disp('>>> OS-CMA: optimal step-size polynomial degeneracy')
   mu_opt = 0; 
else
    
%%% Select optimal root    
    % consider only real-valued roots
    rts = real(rts(abs(imag(rts)) < eps));    % keep only real roots
                                              % (there is at least one real-valued root)
    nrts = length(rts);

    cost = zeros(1, nrts);
    for k = 1:nrts %evaluate cost function a given roots to find the optimal
        cost(k) =  cm_cost(w-rts(k)*g, X, gamma);     
    end
     
    [~, pos_min] = min(cost);
    mu_opt = rts(pos_min); mu_opt = mu_opt(1); % choose the first element, in case there are several 
end % if isempty(rts)

